mMinimap = LibStub("AceAddon-3.0"):NewAddon("mMinimap", "AceEvent-3.0", "AceConsole-3.0")
local mMinimap = mMinimap

-- Locals
local db
local OptionsFrame

-- GUI
local UIF
local LSM = LibStub("LibSharedMedia-3.0")
local AceGUI = LibStub("AceGUI-3.0")

local mMinimapIcon = [[Interface\Icons\INV_Offhand_Sunwell_D_01]]
local MMINIMAP_ANCHOR_POINTS = {
	CENTER = "CENTER",
	TOPLEFT = "TOPLEFT",
	TOPRIGHT = "TOPRIGHT",
	BOTTOMLEFT = "BOTTOMLEFT",
	BOTTOMRIGHT = "BOTTOMRIGHT",
}

local TimeOnEnter, TimeOnLeave
local MouseOverMap
local updateFrame = CreateFrame("Frame")
updateFrame:SetScript("OnUpdate", function(self, elapsed)
	self.elapsed = (self.elapsed or 0) + elapsed
	if self.elapsed > 0.2 then
		if MinimapCoordinates then
			local x, y = GetPlayerMapPosition("player")
			if x and y then
				MinimapCoordinates:SetText(("%.1f, %.1f"):format(x * 100, y * 100))
			end
		end
		self.elapsed = 0
	end	
end)

mMinimap.WidgetRegistry = AceGUI.WidgetRegistry
mMinimap.WidgetVersions = AceGUI.WidgetVersions
mMinimap.WidgetPools = AceGUI.objPools

function mMinimap:OnInitialize()
	local defaults = {
		profile = {
			Enabled = true,
			MoveLFDFrame = true,
			HideTracking = true,
			MinimapLocked = true,
			MinimapScale = 1.5,
			MinimapPositionX = nil,
			MinimapPositionY = nil,
			MinimapAnchor = "UIParent",
			MinimapShape = "SQUARE",
			
			-- Clock
			HideClock = true,
			
			-- Coords
			ShowCoords = true,
			
			-- Mail
			ReplaceMailIcon = true,
			
			-- Font
			ZoneFont = "accid",
			ZoneFontSize = 14,
			ZoneFontFlags = "OUTLINE",
			ZoneFontColor = {1, 1, 1, 1},
			ZoneTextX = 0,
			ZoneTextY = 3,
		}
	}
	
	self.db  = LibStub("AceDB-3.0"):New("mMinimapDB", defaults, "Default")
	db = self.db.profile
	
	self:SetupLDB()
	
	self:SetEnabledState(db.Enabled)
	
	self:RegisterEvent("VARIABLES_LOADED")
end

function mMinimap:OnEnable()
	
	-- Register Events
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA", "NewArea")
	self:RegisterEvent("PLAYER_ENTERING_WORLD", "NewArea")
	self:RegisterEvent("UPDATE_INSTANCE_INFO", "NewArea")
	self:RegisterEvent("CALENDAR_UPDATE_PENDING_INVITES")
	
	-- Register slash command
	self:RegisterChatCommand("mm", "ToggleConfig")
	
	local UIFSettings = {
		s = {
			font = LibStub("LibSharedMedia-3.0"):Fetch("font", "accid"),
			fontSize = 16,			
		}
	}

	UIF = LibStub("LibGUIFactory-1.0"):GetFactory("mMinimap", UIFSettings)
	
	self:ChangeCallback()
	self:NewArea()
end

function mMinimap:OnDisable()
end


function mMinimap:HideDefaultFrames()
	-- Mousewheel zoom
	MinimapZoomIn:Hide()
	MinimapZoomOut:Hide()
	Minimap:EnableMouseWheel()
	Minimap:SetScript('OnMouseWheel', function(self, direction)
		if(direction > 0) then
			MinimapZoomIn:Click()
		else
			MinimapZoomOut:Click()
		end
	end)
	
	-- Tracking
	if db.HideTracking then
		MiniMapTracking:Hide()
		MiniMapTrackingButton:EnableMouse(false)
	else
		MiniMapTracking:Show()		
		MiniMapTrackingButton:EnableMouse(true)
	end
	MiniMapTrackingIconOverlay:SetTexture(nil)
	MiniMapTrackingButton:SetHighlightTexture(nil)
	
	-- LFG
	MiniMapLFGFrameBorder:Hide()	
	
	-- Border
	MinimapBorder:SetTexture(nil)
	MinimapBorderTop:Hide()
	
	-- Compas
	MinimapNorthTag:SetAlpha(0)
	
	-- Instance Difficulty
	MiniMapInstanceDifficulty:Hide()
	
	-- Clock
	TimeManagerClockButton:GetRegions():Hide()
	if db.HideClock then
		TimeManagerClockButton:Hide()
		TimeManagerClockTicker:Hide()
	else
		TimeManagerClockButton:Show()
		TimeManagerClockTicker:Show()
	end
	
	if db.ReplaceMailIcon then
		MiniMapMailIcon:Hide()
		if not MiniMapMailFrame.Text then
			MiniMapMailFrame.Text = MiniMapMailFrame:CreateFontString()
			MiniMapMailFrame.Text:SetPoint("TOP", 0, -2)
			MiniMapMailFrame.Text:SetFont(LSM:Fetch("font", db.ZoneFont), db.ZoneFontSize > 2 and db.ZoneFontSize - 2 or db.ZoneFontSize, db.ZoneFontFlags)
			MiniMapMailFrame.Text:SetText("New Mail!")
		end
		MiniMapMailFrame.Text:SetFont(LSM:Fetch("font", db.ZoneFont), db.ZoneFontSize > 2 and db.ZoneFontSize - 2 or db.ZoneFontSize, db.ZoneFontFlags)
		MiniMapMailFrame.Text:SetTextColor(unpack(db.ZoneFontColor))
		MiniMapMailFrame.Text:Show()
	else
		MiniMapMailIcon:Show()
		if MiniMapMailFrame.Text then
			MiniMapMailFrame.Text:Hide()
		end
	end
	
	if not MinimapCoordinates then
		MinimapCoordinates = Minimap:CreateFontString()		
	end
	
	if db.ShowCoords then				
		MinimapCoordinates:Show()
	else
		MinimapCoordinates:Hide()
	end
	MinimapCoordinates:ClearAllPoints()
	MinimapCoordinates:SetPoint("BOTTOM", 0, 2)
	MinimapCoordinates:SetFont(LSM:Fetch("font", db.ZoneFont), db.ZoneFontSize, db.ZoneFontFlags)
	MinimapCoordinates:SetTextColor(unpack(db.ZoneFontColor))
end


function mMinimap:MoveFrames()
	Minimap:ClearAllPoints()
	local scale = db.MinimapScale
	if not db.MinimapPositionX then
		Minimap:SetPoint("TOPRIGHT", -10, -10)
	else
		Minimap:SetPoint("CENTER", db.MinimapAnchor, "CENTER", db.MinimapPositionX / scale, db.MinimapPositionY / scale)
	end
	Minimap:SetScale(scale)
	
	-- Tracking
	MiniMapTracking:ClearAllPoints()
	MiniMapTracking:SetPoint("TOPLEFT", Minimap, "TOPLEFT")		
		
	-- Looking for Dungeon (Tooltip)
	if db.MoveLFDFrame then
		LFDSearchStatus:ClearAllPoints()
		LFDSearchStatus:SetPoint("CENTER", UIParent)
	else
		LFDSearchStatus:ClearAllPoints()
		LFDSearchStatus:SetPoint("TOPRIGHT", MiniMapLFGFrame, "TOPLEFT")
	end
	
	-- Looking for Dungeon (Icon)
	MiniMapLFGFrame:ClearAllPoints()
	MiniMapLFGFrame:SetPoint("TOP", MiniMapTracking, "BOTTOM")
	
	-- Battleground
	MiniMapBattlefieldFrame:ClearAllPoints()
	MiniMapBattlefieldFrame:SetPoint("TOPRIGHT", Minimap)
	MiniMapBattlefieldBorder:SetTexture(nil)
	
	-- New Mail
	MiniMapMailFrame:ClearAllPoints()
	MiniMapMailFrame:SetPoint("TOP")	
	
	-- World map button
	MiniMapWorldMapButton:Hide()
	
	-- Time/Calendar
	GameTimeFrame:Hide()
	
	-- Zone text
	MinimapZoneText:SetAllPoints(MinimapZoneTextButton)
	MinimapZoneText:SetShadowOffset(0, 0)
	MinimapZoneTextButton:ClearAllPoints()
	MinimapZoneTextButton:SetParent(Minimap)
	MinimapZoneTextButton:SetPoint("BOTTOM", Minimap, "TOP", db.ZoneTextX, db.ZoneTextY)
	MinimapZoneTextButton:SetWidth(Minimap:GetWidth() * 1.5)
	
	-- Instance Difficulty
	MiniMapInstanceDifficulty:ClearAllPoints()
	MiniMapInstanceDifficulty:SetPoint("BOTTOMRIGHT", Minimap)
	
	-- Clock	
	
	TimeManagerClockTicker:SetParent(Minimap)
	TimeManagerClockTicker:ClearAllPoints()
	
	TimeManagerClockTicker:SetPoint("BOTTOMLEFT", 3, 2)
	TimeManagerClockTicker:SetFont(LSM:Fetch("font", db.ZoneFont), db.ZoneFontSize, db.ZoneFontFlags)
	TimeManagerClockTicker:SetTextColor(unpack(db.ZoneFontColor))
	TimeManagerClockTicker:SetShadowOffset(0, 0)
	TimeManagerAlarmFiredTexture.Show = function() TimeManagerClockTicker:SetTextColor(1, 0, 0) TimeManagerClockTicker:Show() end
	TimeManagerAlarmFiredTexture.Hide = function() TimeManagerClockTicker:SetTextColor(1, 1, 1) end
	
	TimeManagerClockButton:ClearAllPoints()
	TimeManagerClockButton:SetAllPoints(TimeManagerClockTicker)
	TimeManagerClockButton:SetScript('OnShow', nil)
	TimeManagerClockButton:Show()
	TimeManagerClockButton:SetScript('OnClick', function(self, button)
		if(button == 'RightButton') then
			ToggleCalendar()
		else
			if(self.alarmFiring) then
				PlaySound('igMainMenuQuit')
				TimeManager_TurnOffAlarm()
			else
				ToggleTimeManager()
			end
		end
	end)
	 
	if not TimeOnEnter then
		TimeOnEnter = TimeManagerClockButton:GetScript("OnEnter")
		TimeManagerClockButton:SetScript("OnEnter", function()
			Minimap:GetScript("OnEnter")()
			TimeOnEnter(TimeManagerClockButton)
		end)
	end
	
	if not TimeOnLeave then
		TimeOnLeave = TimeManagerClockButton:GetScript("OnLeave")
		TimeManagerClockButton:SetScript("OnLeave", function()
			Minimap:GetScript("OnLeave")()
			TimeOnLeave(TimeManagerClockButton)
		end)
	end
	
end


function mMinimap:SetAttributes()

	-- Minimap Shape
	if db.MinimapShape == "SQUARE" then
		Minimap:SetMaskTexture([=[Interface\ChatFrame\ChatFrameBackground]=])
	else
		Minimap:SetMaskTexture([=[Textures\MinimapMask]=])
	end
	
	MiniMapTrackingButtonBorder:Hide()
	MiniMapTrackingIcon:SetTexCoord(0.065, 0.935, 0.065, 0.935)
	
	-- Movable Minimap
	if not db.MinimapLocked then
		Minimap:RegisterForDrag("LeftButton")
		Minimap:SetMovable(true)
		Minimap:SetScript('OnDragStop', function() 
			Minimap:StopMovingOrSizing() 
			local scale = Minimap:GetEffectiveScale() / UIParent:GetEffectiveScale()
			
			x, y = Minimap:GetCenter()
			x, y = x * scale, y * scale
			
			x = x - GetScreenWidth() / 2
			y = y - GetScreenHeight() / 2
			
			db.MinimapPositionX, db.MinimapPositionY = x, y
			
			if OptionsFrame then
				OptionsFrame:ReleaseChildren()
				OptionsFrame:Release()
				OptionsFrame = nil
				mMinimap:ToggleConfig()
			end
		end)
		Minimap:SetScript('OnDragStart', function() 
			Minimap:StartMoving() 
		end)
		MinimapCluster:EnableMouse(false)
		Minimap:SetClampedToScreen(true)
	else
		Minimap:SetMovable(false)
		Minimap:SetScript("OnDragStop", nil)
		Minimap:SetScript("OnDragStart", nil)
	end
	
	MiniMapMailBorder:SetTexture(nil)
	
	-- Zone Text
	MinimapZoneText:SetFont(LSM:Fetch("font", db.ZoneFont), db.ZoneFontSize, db.ZoneFontFlags)
end

---------------------------------------------------
-- Events
---------------------------------------------------
Minimap:SetScript("OnMouseUp", function(self, button)
	if (button == 'RightButton') and  db.HideTracking then
		ToggleDropDownMenu(1, nil, MiniMapTrackingDropDown, self, 0, 0)
	elseif (button == "MiddleButton") then
		ToggleCalendar()
	else
		Minimap_OnClick(self, button)
	end
end)

Minimap:SetScript("OnEnter", function()
	MiniMapTracking:Show()
	if IsInInstance() then
		MiniMapInstanceDifficulty:Show()
	end
	MouseOverMap = true
	TimeManagerClockButton:Show()
	TimeManagerClockTicker:Show()
	MinimapCoordinates:Show()
end)

Minimap:SetScript("OnLeave", function()
	mMinimap:HideDefaultFrames()
	MiniMapInstanceDifficulty:Hide()
	MouseOverMap = false
end)

function mMinimap:NewArea(event)
	if not MouseOverMap then
		MiniMapInstanceDifficulty:Hide()
	end
end

function mMinimap:CALENDAR_UPDATE_PENDING_INVITES()
	if(CalendarGetNumPendingInvites() ~= 0) then
		TimeManagerClockTicker:SetTextColor(0, 1, 0)
	else
		TimeManagerClockTicker:SetTextColor(1, 1, 1)
	end
end

function mMinimap:VARIABLES_LOADED()
	SetCVar('showClock', '1')

	LoadAddOn("Blizzard_TimeManager")
end

---------------------------------------------------
-- Config
---------------------------------------------------
do	
	function mMinimap:ChangeCallback(widget, event, ...)
		if widget and not widget.GetUserData then
			widget = self
			self = mMinimap
		end		
		self:HideDefaultFrames()
		self:MoveFrames()
		self:SetAttributes()
	end

	function mMinimap:ToggleConfig()
		if not OptionsFrame then	
			local m = UIF:Frame("mMinimap")
			m:SetLayout("Fill")
			m:SetWidth(813)
			m:SetHeight(630)
			m:SetCallback("OnClose", function(widget)
				widget:ReleaseChildren()
				widget:Release()
				OptionsFrame = nil
			end)				
			OptionsFrame = m
			
			m = AceGUI:Create("ScrollFrame")			
			m:SetLayout("Flow")
			OptionsFrame:AddChild(m)
			
			local select, slider, color, group, group2, check, text
			group = UIF:InlineGroup("")
			text = UIF:NormalText([[- Right click will show the tracking menu.
- Middle click shows the calendar.
- Hover over to show the clock (and tracking icon if it is hidden).]])
			text:SetImage(mMinimapIcon)
			group:AddChild(text)
			m:AddChild(group)
			
			check = UIF:CheckBox("Lock Minimap", db, "MinimapLocked", mMinimap.ChangeCallback, "While locked, the Minimap cannot be moved with the mouse.")
			check:SetRelativeWidth(0.33)
			m:AddChild(check)
			check = UIF:CheckBox("Move \"Looking For Fungeon\" tooltip", db, "MoveLFDFrame", mMinimap.ChangeCallback, "This will move the tooltip to the middle of the screen")
			check:SetRelativeWidth(0.33)
			m:AddChild(check)
			check = UIF:CheckBox("Hide tracking icon", db, "HideTracking", mMinimap.ChangeCallback, "If hidden right click on the Minimap will show the tracking menu.")
			check:SetRelativeWidth(0.33)
			m:AddChild(check)
			check = UIF:CheckBox("Hide Clock", db, "HideClock", mMinimap.ChangeCallback, "Hide the clock. It will be visible while hovering over the minimap instead.")
			check:SetRelativeWidth(0.33)
			m:AddChild(check)		
			check = UIF:CheckBox("Replace new mail icon", db, "ReplaceMailIcon", mMinimap.ChangeCallback, "Will replace the new mail icon with a text saying the same thing.")
			check:SetRelativeWidth(0.33)
			m:AddChild(check)
			check = UIF:CheckBox("Show Coordinates", db, "ShowCoords", mMinimap.ChangeCallback, "Shows your current coordinates on the botom of the minimap. If hidden it will show on mouseover.")
			check:SetRelativeWidth(0.33)
			m:AddChild(check)		
			
			m:AddChild(UIF:NormalText(" "))
			
			group = UIF:InlineGroup("Position and Scale")
			m:AddChild(group)
			group:AddChild(UIF:Slider("X", db, "MinimapPositionX", 
				-floor(GetScreenWidth() * UIParent:GetEffectiveScale()), 
				floor(GetScreenWidth() * UIParent:GetEffectiveScale()), 0.1, mMinimap.ChangeCallback))
			group:AddChild(UIF:Slider("Y", db, "MinimapPositionY", 
				-floor(GetScreenHeight() * UIParent:GetEffectiveScale()), 
				floor(GetScreenHeight() * UIParent:GetEffectiveScale()), 0.1, mMinimap.ChangeCallback))	
			group:AddChild(UIF:Slider("Scale", db, "MinimapScale", 0.01, 10, 0.01, mMinimap.ChangeCallback))
			
			group = UIF:InlineGroup("Font Settings")
			group:AddChild(UIF:LSMDropdown("font", "Font", db, "ZoneFont", mMinimap.ChangeCallback))
			select = UIF:Dropdown("Outline", db, "ZoneFontFlags", mMinimap.ChangeCallback, {OUTLINE = "Outline", THICKOUTLINE = "Thick Outline", NONE = "None"})
			select:SetWidth(140)
			group:AddChild(select)
			group:AddChild(UIF:Slider("Size", db, "ZoneFontSize", 1, 50, 1, mMinimap.ChangeCallback))	
			group:AddChild(UIF:ColorSelect("Color", db, "ZoneFontColor", mMinimap.ChangeCallback))			
			m:AddChild(group)					
			
			m:AddChild(UIF:Slider("Offset X", db, "ZoneTextX", -50, 50, 0.1, mMinimap.ChangeCallback))
			m:AddChild(UIF:Slider("Offset Y", db, "ZoneTextY", -200, 200, 0.1, mMinimap.ChangeCallback))			
			
			OptionsFrame:PerformLayout()
		
		else
			OptionsFrame:ReleaseChildren()
			OptionsFrame:Release()
			OptionsFrame = nil
		end
	end

	function mMinimap:SetupLDB()
		local LDB = LibStub("LibDataBroker-1.1")
		if not LDB then return end
		
		local l = LDB:NewDataObject("mMinimap")
		l.type = "launcher"
		l.icon = mMinimapIcon
		l.tocname = "mMinimap"
		l.label = "mMinimap"
		l.OnClick = function(self, button)
			mMinimap:ToggleConfig()
		end
		l.OnTooltipShow = function(tt)
			tt:AddLine("mMinimap")
			tt:AddLine("Click to toggle configuration")
		end
		
		self.ldbojb = l
	end
end

function GetMinimapShape() return db.MinimapShape end